"""
Parse xmind to programmable data types.
"""

import json
import logging
import os
import sys
from zipfile import ZipFile

config = {
    "logName": __name__,
    "logLevel": None,
    "logFormat": "%(asctime)s %(levelname)-8s: %(message)s",
    "showTopicId": False,
    "hideEmptyValue": True,
}

cache = {}
_log_name = config["logName"] or __file__
_log_level = config["logLevel"] or logging.WARNING
_log_fmt = config["logFormat"] or "%(asctime)s %(levelname)-8s: %(message)s"

logger = logging.getLogger(_log_name)
logger.setLevel(_log_level)
console_handler = logging.StreamHandler(sys.stdout)
console_handler.setFormatter(logging.Formatter(config["logFormat"]))
logger.addHandler(console_handler)


def set_logger_level(new_level):
    logger.setLevel(new_level)


def is_xmind_zen(file_path):
    """Determine if this is a xmind zen file type."""
    with ZipFile(file_path) as xmind:
        return "content.json" in xmind.namelist()


def get_xmind_zen_builtin_json(file_path):
    """Read internal content.json from xmind zen file."""
    name = "content.json"
    with ZipFile(file_path) as xmind:
        if name in xmind.namelist():
            content = xmind.open(name).read().decode("utf-8")
            return json.loads(content)

        raise AssertionError("Not a xmind zen file type!")


def _get_out_file_name(xmind_file, suffix):
    assert isinstance(xmind_file, str) and xmind_file.endswith(".xmind"), "Invalid xmind file!"
    name = os.path.abspath(xmind_file[0:-5] + suffix)

    return name


def xmind_to_dict(file_path):
    """Open and convert xmind to dict type."""
    if is_xmind_zen(file_path):
        from .zenreader import get_sheets, open_xmind, sheet_to_dict
    else:
        from .xreader import get_sheets, open_xmind, sheet_to_dict

    open_xmind(file_path)
    data = []

    for s in get_sheets():
        data.append(sheet_to_dict(s))

    return data


def xmind_to_file(file_path, file_type):
    if file_type == "json":
        return xmind_to_json(file_path)

    elif file_type == "xml":
        return xmind_to_xml(file_path)

    else:
        raise ValueError("Not supported file type: {}".format(file_type))


def xmind_to_json(file_path):
    target = _get_out_file_name(file_path, "json")

    with open(target, "w", encoding="utf8") as f:
        f.write(json.dumps(xmind_to_dict(file_path), indent=2))

    return target


def xmind_to_xml(file_path):
    try:
        from xml.dom.minidom import parseString

        from dicttoxml import dicttoxml

        target = _get_out_file_name(file_path, "xml")
        xml = dicttoxml(xmind_to_dict(file_path), custom_root="root")
        xml = parseString(xml.decode("utf8")).toprettyxml(encoding="utf8")

        with open(target, "w", encoding="utf8") as f:
            f.write(xml.decode("utf8"))

        return target
    except ImportError:
        raise ImportError('Parse xmind to xml require "dicttoxml", try install via pip:\n' + "> pip install dicttoxml")



#!/usr/bin/env python
# _*_ coding:utf-8 _*_
import logging
import os
from io import BytesIO
from xml.dom import minidom
from xml.etree.ElementTree import Comment, Element, ElementTree, SubElement
from xml.sax.saxutils import escape

from xmind2testcasezen import const
from xmind2testcasezen.const import config
from xmind2testcasezen.services import get_absolute_path, get_raw_testsuites

"""
Convert XMind fie to TestLink testcase xml file 
"""


def xmind_to_testlink_xml_file(xmind_file, is_all_sheet=True):
    """Convert a XMind sheet to a testlink xml file"""
    xmind_file = get_absolute_path(xmind_file)
    logging.info("Start converting XMind file(%s) to testlink file...", xmind_file)
    testsuites = get_raw_testsuites(xmind_file)
    if not is_all_sheet and testsuites:
        testsuites = [testsuites[0]]

    xml_content = testsuites_to_xml_content(testsuites)
    testlink_xml_file = xmind_file[:-6] + ".xml"

    if os.path.exists(testlink_xml_file):
        logging.info("the testlink xml file already exists, return it directly: %s", testlink_xml_file)
        return testlink_xml_file

    with open(testlink_xml_file, "w", encoding="utf-8") as f:
        pretty_content = minidom.parseString(xml_content).toprettyxml(indent="\t")
        f.write(pretty_content)
        logging.info("convert XMind file(%s) to a testlink xml file(%s) successfully!", xmind_file, testlink_xml_file)

    return testlink_xml_file


def testsuites_to_xml_content(testsuites):
    """Convert the testsuites to testlink xml file format"""
    root_element = Element(const.TAG_TESTSUITE)
    # setting the root suite's name attribute, that will generate a new testsuite folder on testlink
    # root_element.set(const.ATTR_NMAE, testsuite.name)

    for testsuite in testsuites:
        suite_element = SubElement(root_element, const.TAG_TESTSUITE)
        suite_element.set(const.ATTR_NMAE, testsuite.name)
        gen_text_element(suite_element, const.TAG_DETAILS, testsuite.details)

        for sub_suite in testsuite.sub_suites:
            if is_should_skip(sub_suite.name):
                continue
            sub_suite_element = SubElement(suite_element, const.TAG_TESTSUITE)
            sub_suite_element.set(const.ATTR_NMAE, sub_suite.name)
            gen_text_element(sub_suite_element, const.TAG_DETAILS, sub_suite.details)
            gen_testcase_element(sub_suite_element, sub_suite)

    testlink = ElementTree(root_element)
    content_stream = BytesIO()
    testlink.write(content_stream, encoding="utf-8", xml_declaration=True)
    return content_stream.getvalue()


def gen_testcase_element(suite_element, suite):
    for testcase in suite.testcase_list:
        if is_should_skip(testcase.name):
            continue

        testcase_elment = SubElement(suite_element, const.TAG_TESTCASE)
        testcase_elment.set(const.ATTR_NMAE, testcase.name)

        gen_text_element(testcase_elment, const.TAG_VERSION, str(testcase.version))
        gen_text_element(testcase_elment, const.TAG_SUMMARY, testcase.summary)
        gen_text_element(testcase_elment, const.TAG_PRECONDITIONS, testcase.preconditions)
        gen_text_element(testcase_elment, const.TAG_EXECUTION_TYPE, _convert_execution_type(testcase.execution_type))
        gen_text_element(testcase_elment, const.TAG_IMPORTANCE, _convert_importance(testcase.importance))

        estimated_exec_duration_element = SubElement(testcase_elment, const.TAG_ESTIMATED_EXEC_DURATION)
        estimated_exec_duration_element.text = str(testcase.estimated_exec_duration)

        status = SubElement(testcase_elment, const.TAG_STATUS)
        status.text = str(testcase.status) if testcase.status in (1, 2, 3, 4, 5, 6, 7) else "7"

        gen_steps_element(testcase_elment, testcase)


def gen_steps_element(testcase_element, testcase):
    if testcase.steps:
        steps_element = SubElement(testcase_element, const.TAG_STEPS)

        for step in testcase.steps:
            if is_should_skip(step.actions):
                continue

            step_element = SubElement(steps_element, const.TAG_STEP)
            gen_text_element(step_element, const.TAG_STEP_NUMBER, str(step.step_number))
            gen_text_element(step_element, const.TAG_ACTIONS, step.actions)
            gen_text_element(step_element, const.TAG_EXPECTEDRESULTS, step.expectedresults)
            gen_text_element(step_element, const.TAG_EXECUTION_TYPE, _convert_execution_type(step.execution_type))


def gen_text_element(parent_element, tag_name, content):
    """generate an element's text conent: <![CDATA[text]]>"""
    if is_should_parse(content):
        child_element = SubElement(parent_element, tag_name)
        element_set_text(child_element, content)


def element_set_text(element, content):
    # retain html tags in content
    content = escape(content, entities={"\r\n": "<br />"})
    # replace new line for *nix system
    content = content.replace("\n", "<br />")
    # add the line break in source to make it readable
    content = content.replace("<br />", "<br />\n")

    # add CDATA for a element
    element.append(Comment(" --><![CDATA[" + content.replace("]]>", "]]]]><![CDATA[>") + "]]> <!-- "))


def is_should_parse(content):
    """An element that has a string content and doesn't start with exclamation mark should be parsing"""
    return isinstance(content, str) and content.strip() != "" and not content[0] in config["ignore_char"]


def is_should_skip(content):
    """A testsuite/testcase/teststep should be skip: 1、content is empty; 2、starts with config.ignore_char"""
    return (
        content is None or not isinstance(content, str) or content.strip() == "" or content[0] in config["ignore_char"]
    )


def _convert_execution_type(value):
    if value in (1, "手动", "手工", "manual", "Manual"):
        return "1"
    elif value in (2, "自动", "自动化", "自动的", "Automate", "Automated", "Automation", "automate", "automated", "automation"):
        return "2"
    else:
        return "1"


def _convert_importance(value):
    mapping = {1: "3", 2: "2", 3: "1"}
    if value in mapping.keys():
        return mapping[value]
    else:
        return "2"


if __name__ == "__main__":
    xmind_file = "../docs/xmind_testcase_template.xmind"
    testlink_xml_file = xmind_to_testlink_xml_file(xmind_file)
    print("Convert XMind file to testlink xml file successfully: %s", testlink_xml_file)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Time:2023/3/1 21:00
# @Author:boyizhang
import logging

from xmind2testcasezen import const
from xmind2testcasezen.metadata import AttachedTopicAttribute, TestCase, TestStep, TestSuite
from xmind2testcasezen.parser_xmind import ParseXmind


class ParseNew(ParseXmind):
    def parse_testsuites(self, xmind_content_dict):
        """convert xmind file to `xmind2testcasezen.metadata.TestSuite` list"""
        suites = []

        for sheet in xmind_content_dict:
            logging.debug("start to parse a sheet: %s", sheet["title"])
            root_topic = AttachedTopicAttribute(sheet["rootTopic"])
            # sub_topics = AttachedTopicAttribute(root_topic.sub_attached_topics)
            suite = self.parse_sub_testsuite(root_topic)
            suites.append(suite)

        return suites

    def parse_sub_testsuite(self, root_topic: AttachedTopicAttribute):
        suite = TestSuite()
        suite.name = root_topic.title
        suite.details = root_topic.note
        suite.sub_suites = []

        for suite_dict in root_topic.sub_attached_topics:
            suite.sub_suites.append(self.parse_testsuite(suite_dict))
        return suite

    def parse_testsuite(self, suite_dict):
        testsuite = TestSuite()
        suite_attached_topic = AttachedTopicAttribute(suite_dict)
        testsuite.name = suite_attached_topic.title
        testsuite.details = suite_attached_topic.note
        testsuite.testcase_list = []
        # sub_attached_topics：层级2
        for cases_dict in suite_attached_topic.sub_attached_topics:
            for case in self.recurse_parse_testcase(cases_dict):
                testsuite.testcase_list.append(case)

        return testsuite

    def recurse_parse_testcase(self, case_dict, parent=[]):
        cur_attached_topic = AttachedTopicAttribute(case_dict)
        if cur_attached_topic.is_testcase:
            case = self.parse_a_testcase(cur_attached_topic, parent)
            yield case
        else:
            # if not parent:
            #     parent = []
            parent.append(cur_attached_topic)

            for child_dict in cur_attached_topic.sub_attached_topics:
                for case in self.recurse_parse_testcase(child_dict, parent):
                    yield case

            parent.pop()

    def parse_a_testcase(self, cur_attached_topic, parent):
        testcase = TestCase()
        attached_topics = parent + [cur_attached_topic] if parent else [cur_attached_topic]
        testcase.name = cur_attached_topic.title
        testcase.category = self.gen_testcase_category(attached_topics)
        testcase.summary = cur_attached_topic.note
        testcase.importance = cur_attached_topic.importance
        testcase.execution_type = cur_attached_topic.execution_type
        preconditions = self.gen_testcase_preconditions(cur_attached_topic.sub_attached_topics)
        testcase.preconditions = preconditions if preconditions else ""
        testcase.steps = self.parse_test_steps(cur_attached_topic.sub_attached_topics)
        return testcase

    def gen_testcase_category(self, attached_topics):
        """Link all topic's title as testcase title"""
        # 去掉testcase节点
        titles = [f"<b>{attached_topic.title}</b>" for attached_topic in attached_topics[:-1]]
        # when separator is not blank, will add space around separator, e.g. '/' will be changed to ' / '
        separator = const.config["sep"]
        # logging.info(f'bseparator:{separator}')
        # if separator != ' > ':
        #     separator = ' {} > '.format(separator)
        return separator.join(titles)

    def gen_testcase_preconditions(self, attached_topics):
        preconditions = []
        for attached_topic_dict in attached_topics:
            precondition_attached_topic = AttachedTopicAttribute(attached_topic_dict)
            if precondition_attached_topic.is_precondition:
                sub_attached_topics = precondition_attached_topic.sub_attached_topics
                if sub_attached_topics or len(sub_attached_topics) > 0:
                    # 如果包含子节点，则取子节点的title
                    for sub_attached_topic_dict in sub_attached_topics:
                        sub_attached_topic = AttachedTopicAttribute(sub_attached_topic_dict)
                        preconditions.append(sub_attached_topic.title)
                else:
                    preconditions.append(precondition_attached_topic.title)
                return const.config["precondition_sep"].join(preconditions)

        return preconditions

    def parse_test_steps(self, attached_topics):
        steps = []
        attached_step_topics = []
        for attached_topic in attached_topics:
            cur_attached_step_topic = AttachedTopicAttribute(attached_topic)
            if not cur_attached_step_topic.is_teststep:
                continue
            attached_step_topics.append(cur_attached_step_topic)

        for step_num, cur_attached_step_topic in enumerate(attached_step_topics, start=1):
            if not cur_attached_step_topic.is_teststep:
                continue
            test_step = self.parse_a_test_step(cur_attached_step_topic)
            test_step.step_number = step_num
            steps.append(test_step)

        return steps

    def parse_a_test_step(self, cur_attached_step_topic):
        test_step = TestStep()
        test_step.actions = cur_attached_step_topic.title

        expected_topics = cur_attached_step_topic.sub_attached_topics
        expected_results = []
        if expected_topics:  # have expected result
            for expected_topic_dict in expected_topics:
                expected_topic = AttachedTopicAttribute(expected_topic_dict)
                if not expected_topic.is_expect_result:
                    continue
                expected_results.append(expected_topic.title)  # one test step action, one test expected result
                # markers = expected_topic.markers

        test_step.expectedresults = expected_results

        logging.debug("finds a teststep: %s", test_step.to_dict())
        return test_step



#!/usr/bin/env python
# _*_ coding:utf-8 _*_
import logging
import sys

from webtool.application import launch

from xmind2testcasezen.services import gen_testcase_to_json_file
from xmind2testcasezen.testlink import xmind_to_testlink_xml_file
from xmind2testcasezen.utils import get_absolute_path
from xmind2testcasezen.zentao import xmind_to_zentao_csv_file

logging.basicConfig(
    level=logging.INFO,
    format="%(asctime)s  %(name)s  %(levelname)s  [%(module)s - %(funcName)s]: %(message)s",
    datefmt="%Y/%m/%d %H:%M:%S",
)

using_doc = """
    Xmind2Testcase is a tool to parse xmind file into testcase file, which will help you generate a testlink recognized
    xml file or a zentao recognized cvs file, then you can import it into testlink or zentao.
    
    Usage:
     xmind2testcasezen [path_to_xmind_file] [-csv] [-xml] [-json]
     xmind2testcasezen [webtool] [port_num]
    
    Example:
     xmind2testcasezen /path/to/testcase.xmind        => output testcase.csv、testcase.xml、testcase.json
     xmind2testcasezen /path/to/testcase.xmind -csv   => output testcase.csv
     xmind2testcasezen /path/to/testcase.xmind -xml   => output testcase.xml
     xmind2testcasezen /path/to/testcase.xmind -json  => output testcase.json
     xmind2testcasezen webtool                        => launch the web testcase conversion tool locally: 127.0.0.1:5001
     xmind2testcasezen webtool 8000                   => launch the web testcase conversion tool locally: 127.0.0.1:8000
    """


def cli_main():
    if len(sys.argv) > 1 and sys.argv[1].endswith(".xmind"):
        xmind_file = sys.argv[1]
        xmind_file = get_absolute_path(xmind_file)
        logging.info("Start to convert XMind file: %s", xmind_file)

        if len(sys.argv) == 3 and sys.argv[2] == "-json":
            testlink_json_file = gen_testcase_to_json_file(xmind_file)
            logging.info("Convert XMind file to testcase json file successfully: %s", testlink_json_file)
        elif len(sys.argv) == 3 and sys.argv[2] == "-xml":
            testlink_xml_file = xmind_to_testlink_xml_file(xmind_file)
            logging.info("Convert XMind file to testlink xml files successfully: %s", testlink_xml_file)
        elif len(sys.argv) == 3 and sys.argv[2] == "-csv":
            zentao_csv_file = xmind_to_zentao_csv_file(xmind_file)
            logging.info("Convert XMind file to zentao csv file successfully: %s", zentao_csv_file)
        else:
            testlink_json_file = gen_testcase_to_json_file(xmind_file)
            testlink_xml_file = xmind_to_testlink_xml_file(xmind_file)
            zentao_csv_file = xmind_to_zentao_csv_file(xmind_file)
            logging.info(
                "Convert XMind file successfully: \n"
                "1、 testcase json file(%s)\n"
                "2、 testlink xml file(%s)\n"
                "3、 zentao csv file(%s)",
                testlink_json_file,
                testlink_xml_file,
                zentao_csv_file,
            )
    elif len(sys.argv) > 1 and sys.argv[1] == "webtool":
        if len(sys.argv) == 3:
            try:
                port = int(sys.argv[2])
                launch(port=port)
            except ValueError:
                launch()
        else:
            launch()

    else:
        print(using_doc)


if __name__ == "__main__":
    cli_main()



#!/usr/bin/env python
# _*_ coding:utf-8 _*_
import csv
import logging
import os

from xmind2testcasezen.services import get_testcase_list
from xmind2testcasezen.utils import get_absolute_path

"""
Convert XMind fie to Zentao testcase csv file 

Zentao official document about import CSV testcase file: https://www.zentao.net/book/zentaopmshelp/243.mhtml 
"""


def xmind_to_zentao_csv_file(xmind_file):
    """Convert XMind file to a zentao csv file"""
    xmind_file = get_absolute_path(xmind_file)
    logging.info("Start converting XMind file(%s) to zentao file...", xmind_file)
    testcases = get_testcase_list(xmind_file)

    fileheader = ["所属模块", "用例标题", "前置条件", "步骤", "预期", "关键词", "优先级", "用例类型", "适用阶段"]
    zentao_testcase_rows = [fileheader]
    for testcase in testcases:
        row = gen_a_testcase_row(testcase)
        zentao_testcase_rows.append(row)

    zentao_file = xmind_file[:-6] + ".csv"
    if os.path.exists(zentao_file):
        os.remove(zentao_file)
        # logging.info('The zentao csv file already exists, return it directly: %s', zentao_file)
        # return zentao_file

    with open(zentao_file, "w", encoding="utf8") as f:
        writer = csv.writer(f)
        writer.writerows(zentao_testcase_rows)
        logging.info("Convert XMind file(%s) to a zentao csv file(%s) successfully!", xmind_file, zentao_file)

    return zentao_file


def gen_a_testcase_row(testcase_dict):
    case_module = gen_case_module(testcase_dict["suite"])
    case_title = testcase_dict["name"]
    case_precontion = testcase_dict["preconditions"]
    case_step, case_expected_result = gen_case_step_and_expected_result(testcase_dict["steps"])
    case_keyword = ""
    case_priority = gen_case_priority(testcase_dict["importance"])
    case_type = gen_case_type(testcase_dict["execution_type"])
    case_apply_phase = "迭代测试"
    row = [
        case_module,
        case_title,
        case_precontion,
        case_step,
        case_expected_result,
        case_keyword,
        case_priority,
        case_type,
        case_apply_phase,
    ]
    return row


def gen_case_module(module_name):
    if module_name:
        module_name = module_name.replace("（", "(")
        module_name = module_name.replace("）", ")")
    else:
        module_name = "/"
    return module_name


def gen_case_step_and_expected_result(steps):
    case_step = ""
    case_expected_result = ""

    for step_dict in steps:
        case_step += str(step_dict["step_number"]) + ". " + step_dict["actions"].replace("\n", "").strip() + "\n"
        case_expected_result += (
            str(step_dict["step_number"])
            + ". "
            + "------".join(step_dict["expectedresults"]).replace("\n", "").strip()
            + "\n"
            if step_dict.get("expectedresults", "")
            else ""
        )

    return case_step, case_expected_result


def gen_case_priority(priority):
    mapping = {1: "高", 2: "中", 3: "低"}
    if priority in mapping.keys():
        return mapping[priority]
    else:
        return "中"


def gen_case_type(case_type):
    mapping = {1: "手动", 2: "自动"}
    if case_type in mapping.keys():
        return mapping[case_type]
    else:
        return "手动"


if __name__ == "__main__":
    xmind_file = "../docs/zentao_testcase_template.xmind"
    zentao_csv_file = xmind_to_zentao_csv_file(xmind_file)
    print("Conver the xmind file to a zentao csv file succssfully: %s", zentao_csv_file)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Time:2023/3/1 18:34
# @Author:boyizhang
from abc import ABCMeta, abstractmethod


class ParseXmind(metaclass=ABCMeta):
    @abstractmethod
    def parse_testsuites(self, xmind_content_dict):
        raise NotImplementedError



import re
from xml.etree import ElementTree as ET
from xml.etree.ElementTree import Element
from zipfile import ZipFile

from . import cache, config, logger

content_xml = "content.xml"
comments_xml = "comments.xml"


def open_xmind(file_path):
    """open xmind as zip file and cache the content."""
    cache.clear()
    with ZipFile(file_path) as xmind:
        for f in xmind.namelist():
            for key in [content_xml, comments_xml]:
                if f == key:
                    cache[key] = xmind.open(f).read().decode("utf-8")


def get_sheets():
    """get all sheet as generator and yield."""
    tree = xmind_content_to_etree(cache[content_xml])
    assert isinstance(tree, Element)

    for sheet in tree.findall("sheet"):
        yield sheet


def sheet_to_dict(sheet):
    """convert a sheet to dict type."""
    topic = sheet.find("topic")
    result = {"title": title_of(sheet), "topic": node_to_dict(topic), "structure": get_sheet_structure(sheet)}

    if config["showTopicId"]:
        result["id"] = sheet.attrib["id"]

    if config["hideEmptyValue"]:
        result = {k: v for k, v in result.items() if v}

    return result


def get_sheet_structure(sheet):
    root_topic = sheet.find("topic")
    return root_topic.attrib.get("structure-class", None)


def node_to_dict(node):
    """parse Element to dict data type."""
    child = children_topics_of(node)

    d = {
        "title": title_of(node),
        "comment": comments_of(node),
        "note": note_of(node),
        "makers": maker_of(node),
        "labels": labels_of(node),
        "link": link_of(node),
    }

    if d["link"]:
        if d["link"].startswith("xmind"):
            d["link"] = "[To another xmind topic!]"

        if d["link"].startswith("xap:attachments"):
            del d["link"]
            d["title"] = "[Attachment]{0}".format(d["title"])

    if child:
        d["topics"] = []
        for c in child:
            d["topics"].append(node_to_dict(c))

    if config["showTopicId"]:
        d["id"] = id_of(node)

    if config["hideEmptyValue"]:
        d = {k: v for k, v in d.items() if v or k == "title"}

    return d


def xmind_content_to_etree(content):
    # Remove the default namespace definition (xmlns="http://some/namespace")
    xml_content = re.sub(r'\sxmlns="[^"]+"', "", content, count=1)

    # Replace xml tag with namespace
    xml_content = xml_content.replace("<xhtml:img", "<img")

    # Replace link attrib with namespace
    xml_content = xml_content.replace("xlink:href", "href")
    return ET.fromstring(xml_content.encode("utf-8"))


def xmind_xml_to_etree(xml_path):
    with open(xml_path) as f:
        content = f.read()
        return xmind_content_to_etree(content)


def comments_of(node):
    if cache.get(comments_xml, None):
        node_id = node.attrib.get("id", None)

        if node_id:
            xml_root = xmind_content_to_etree(cache[comments_xml])
            result = []

            for c in xml_root.findall("comment"):
                if c.attrib["object-id"] == node_id:
                    i = {"author": c.attrib["author"], "content": c.find("content").text}

                    if config["showTopicId"]:
                        i["id"] = c.attrib["object-id"]

                    result.append(i)

            return result if result else None


def id_of(node):
    return node.attrib.get("id", None)


def image_of(node):
    img = node.find("img")

    if img is not None:
        return "[Image]"


def link_of(node):
    return node.attrib.get("href", None)


def title_of(node):
    if image_of(node):
        return image_of(node)

    title = node.find("title")

    if title is not None:
        return title.text


def labels_of(node):
    label_node = node.find("labels")

    if label_node is not None:
        labels = []
        for _ in label_node.findall("label"):
            labels.append(_.text)

        return labels if labels else None


def note_of(node):
    note_node = node.find("notes")

    if note_node is not None:
        note = note_node.find("plain").text
        return note.strip()


def debug_node(node, comments):
    s = ET.tostring(node)
    logger.debug("{}: {}".format(comments, s))
    return s


def maker_of(topic_node):
    maker_node = topic_node.find("marker-refs")
    if maker_node is not None:
        makers = []
        for maker in maker_node:
            makers.append(maker.attrib["marker-id"])

        return makers


def children_topics_of(topic_node):
    children = topic_node.find("children")

    if children is not None:
        return children.find('./topics[@type="attached"]')



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Time:2023/3/1 21:00
# @Author:boyizhang
import logging

from xmind2testcasezen import const
from xmind2testcasezen.metadata import TestCase, TestStep, TestSuite
from xmind2testcasezen.parser_xmind import ParseXmind


class ParseOld(ParseXmind):
    def get_raw_testsuites(self, xmind_file):
        pass

    def parse_testsuites(self, xmind_content_dict):
        """convert xmind file to `xmind2testcasezen.metadata.TestSuite` list"""
        suites = []

        for sheet in xmind_content_dict:
            logging.debug("start to parse a sheet: %s", sheet["title"])
            root_topic = sheet["topic"]
            sub_topics = root_topic.get("topics", [])

            if sub_topics:
                root_topic["topics"] = self.filter_empty_or_ignore_topic(sub_topics)
            else:
                logging.warning(
                    "This is a blank sheet(%s), should have at least 1 sub topic(test suite)", sheet["title"]
                )
                continue
            suite = self.sheet_to_suite(root_topic)
            # suite.sheet_name = sheet['title']  # root testsuite has a sheet_name attribute
            logging.debug("sheet(%s) parsing complete: %s", sheet["title"], suite.to_dict())
            suites.append(suite)

        return suites

    def filter_empty_or_ignore_topic(self, topics):
        """filter blank or start with const.config.ignore_char topic"""
        result = [
            topic
            for topic in topics
            if not (
                topic["title"] is None
                or topic["title"].strip() == ""
                or topic["title"][0] in const.config["ignore_char"]
            )
        ]

        for topic in result:
            sub_topics = topic.get("topics", [])
            topic["topics"] = self.filter_empty_or_ignore_topic(sub_topics)

        return result

    def filter_empty_or_ignore_element(self, values):
        """Filter all empty or ignore XMind elements, especially notes、comments、labels element"""
        result = []
        for value in values:
            if isinstance(value, str) and not value.strip() == "" and not value[0] in const.config["ignore_char"]:
                result.append(value.strip())
        return result

    def sheet_to_suite(self, root_topic):
        """convert a xmind sheet to a `TestSuite` instance"""
        suite = TestSuite()
        root_title = root_topic["title"]
        separator = root_title[-1]

        if separator in const.config["valid_sep"]:
            logging.debug("find a valid separator for connecting testcase title: %s", separator)
            const.config["sep"] = separator  # set the separator for the testcase's title
            root_title = root_title[:-1]
        else:
            const.config["sep"] = " "

        suite.name = root_title
        suite.details = root_topic["note"]
        suite.sub_suites = []

        for suite_dict in root_topic["topics"]:
            suite.sub_suites.append(self.parse_testsuite(suite_dict))

        return suite

    def parse_testsuite(self, suite_dict):
        testsuite = TestSuite()
        testsuite.name = suite_dict["title"]
        testsuite.details = suite_dict["note"]
        testsuite.testcase_list = []
        logging.debug("start to parse a testsuite: %s", testsuite.name)

        for cases_dict in suite_dict.get("topics", []):
            for case in self.recurse_parse_testcase(cases_dict):
                testsuite.testcase_list.append(case)

        logging.debug("testsuite(%s) parsing complete: %s", testsuite.name, testsuite.to_dict())
        return testsuite

    def recurse_parse_testcase(self, case_dict, parent=None):
        if self.is_testcase_topic(case_dict):
            case = self.parse_a_testcase(case_dict, parent)
            yield case
        else:
            if not parent:
                parent = []

            parent.append(case_dict)

            for child_dict in case_dict.get("topics", []):
                for case in self.recurse_parse_testcase(child_dict, parent):
                    yield case

            parent.pop()

    def is_testcase_topic(self, case_dict):
        """A topic with a priority marker, or no subtopic, indicates that it is a testcase"""
        priority = self.get_priority(case_dict)
        if priority:
            return True

        children = case_dict.get("topics", [])
        if children:
            return False

        return True

    def parse_a_testcase(self, case_dict, parent):
        testcase = TestCase()
        topics = parent + [case_dict] if parent else [case_dict]

        testcase.name = self.gen_testcase_title(topics)

        preconditions = self.gen_testcase_preconditions(topics)
        testcase.preconditions = preconditions if preconditions else "无"

        summary = self.gen_testcase_summary(topics)
        testcase.summary = summary if summary else testcase.name
        testcase.execution_type = self.get_execution_type(topics)
        testcase.importance = self.get_priority(case_dict) or 2

        step_dict_list = case_dict.get("topics", [])
        if step_dict_list:
            testcase.steps = self.parse_test_steps(step_dict_list)

        # the result of the testcase take precedence over the result of the teststep
        testcase.result = self.get_test_result(case_dict["markers"])

        if testcase.result == 0 and testcase.steps:
            for step in testcase.steps:
                if step.result == 2:
                    testcase.result = 2
                    break
                if step.result == 3:
                    testcase.result = 3
                    break

                testcase.result = step.result  # there is no need to judge where test step are ignored

        logging.debug("finds a testcase: %s", testcase.to_dict())
        return testcase

    def get_execution_type(self, topics):
        labels = [topic.get("label", "") for topic in topics]
        labels = self.filter_empty_or_ignore_element(labels)
        exe_type = 1
        for item in labels[::-1]:
            if item.lower() in ["自动", "auto", "automate", "automation"]:
                exe_type = 2
                break
            if item.lower() in ["手动", "手工", "manual"]:
                exe_type = 1
                break
        return exe_type

    def get_priority(self, case_dict):
        """Get the topic's priority（equivalent to the importance of the testcase)"""
        if isinstance(case_dict["markers"], list):
            for marker in case_dict["markers"]:
                if marker.startswith("priority"):
                    return int(marker[-1])

    def gen_testcase_title(self, topics):
        """Link all topic's title as testcase title"""
        titles = [topic["title"] for topic in topics]
        titles = self.filter_empty_or_ignore_element(titles)

        # when separator is not blank, will add space around separator, e.g. '/' will be changed to ' / '
        separator = const.config["sep"]
        if separator != " ":
            separator = " {} ".format(separator)

        return separator.join(titles)

    def gen_testcase_preconditions(self, topics):
        notes = [topic["note"] for topic in topics]
        notes = self.filter_empty_or_ignore_element(notes)
        return const.config["precondition_sep"].join(notes)

    def gen_testcase_summary(self, topics):
        comments = [topic["comment"] for topic in topics]
        comments = self.filter_empty_or_ignore_element(comments)
        return const.config["summary_sep"].join(comments)

    def parse_test_steps(self, step_dict_list):
        steps = []

        for step_num, step_dict in enumerate(step_dict_list, 1):
            test_step = self.parse_a_test_step(step_dict)
            test_step.step_number = step_num
            steps.append(test_step)

        return steps

    def parse_a_test_step(self, step_dict):
        test_step = TestStep()
        test_step.actions = step_dict["title"]
        expected_results = []
        expected_topics = step_dict.get("topics", [])
        if expected_topics:  # have expected result
            for expected_topic in expected_topics:
                expected_results.append(expected_topic["title"])  # one test step action, one test expected result
                markers = expected_topic["markers"]
                test_step.result = self.get_test_result(markers)
            test_step.expectedresults = expected_results
        else:  # only have test step
            markers = step_dict["markers"]
            test_step.result = self.get_test_result(markers)

        logging.debug("finds a teststep: %s", test_step.to_dict())
        return test_step

    def get_test_result(self, markers):
        """test result: non-execution:0, pass:1, failed:2, blocked:3, skipped:4"""
        if isinstance(markers, list):
            if "symbol-right" in markers or "c_simbol-right" in markers:
                result = 1
            elif "symbol-wrong" in markers or "c_simbol-wrong" in markers:
                result = 2
            elif "symbol-pause" in markers or "c_simbol-pause" in markers:
                result = 3
            elif "symbol-minus" in markers or "c_simbol-minus" in markers:
                result = 4
            else:
                result = 0
        else:
            result = 0

        return result



__title__ = "xmind2testcasezen"
__description__ = "XMind2TestCaseZen基于XMind2TestCase做二次开发，提供了一个高效测试用例设计的解决方案！"
__keywords__ = ["xmind2testCase, testcase, test, testing, xmind, 思维导图, XMind思维导图"]
__url__ = "https://github.com/maiyajj/xmind2testcasezen"
__author__ = "maiyajj"
__author_email__ = ""
__version__ = "0.0.3"
__license__ = "MIT"



#!/usr/bin/env python
# _*_ coding:utf-8 _*_
import json
import os
import zipfile

import xmind

from xmind2testcasezen import const, xmind_to_dict


def parser_xmind_to_dict(path="demo2.xmind"):
    input_source = get_absolute_path(path)
    xmind_content_json_dict = {}
    xmind_content_xml_dict = {}

    with extract(input_source) as input_stream:
        for stream in input_stream.namelist():
            # 如果存在content.json文件，使用该文件进行解析并返回
            if stream == "content.json":
                xmind_content_json_dict = json.loads(input_stream.open(stream).read())
                # 使用的是“测试用例”lebels标记用例使用新方法
                if recurse_key_search(xmind_content_json_dict, "labels", const.TESTCASE_TAG):
                    return const.TAG_JSON, xmind_content_json_dict
                else:
                    return const.TAG_PRO, xmind_to_dict(input_source)
    if not xmind_content_json_dict:
        workbook = xmind.load(input_source)
        xmind_content_xml_dict = workbook.getData()

    return const.TAG_XML, xmind_content_xml_dict


def extract(path):
    return zipfile.ZipFile(path, "r")


def get_absolute_path(path):
    """
    Return the absolute path of a file

    If path contains a start point (eg Unix '/') then use the specified start point
    instead of the current working directory. The starting point of the file path is
    allowed to begin with a tilde "~", which will be replaced with the user's home directory.
    """
    fp, fn = os.path.split(path)
    if not fp:
        fp = os.getcwd()
    fp = os.path.abspath(os.path.expanduser(fp))
    return os.path.join(fp, fn)


def recurse_key_search(data, target_key, target_value):
    if isinstance(data, dict):
        for key, value in data.items():
            if key == target_key and target_value in value:
                return value
            elif isinstance(value, (dict, list)):
                result = recurse_key_search(value, target_key, target_value)
                if result is not None:
                    return result
    elif isinstance(data, list):
        for item in data:
            if isinstance(item, (dict, list)):
                result = recurse_key_search(item, target_key, target_value)
                if result is not None:
                    return result
    return None



#!/usr/bin/env python
# _*_ coding:utf-8 _*_
import logging
import os
import re
import sqlite3
from contextlib import closing
from os.path import exists, join

import arrow
from flask import Flask, abort, g, redirect, render_template, request, send_from_directory, url_for
from werkzeug.utils import secure_filename

# from xmind2testcasezen.utils import get_xmind_testsuites, get_xmind_testcase_list
from xmind2testcasezen.services import count_testsuits, get_testcase_list
from xmind2testcasezen.testlink import xmind_to_testlink_xml_file
from xmind2testcasezen.zentao import xmind_to_zentao_csv_file

here = os.path.abspath(os.path.dirname(__file__))
log_file = os.path.join(here, "running.log")
# log handler
formatter = logging.Formatter("%(asctime)s  %(name)s  %(levelname)s  [%(module)s - %(funcName)s]: %(message)s")
file_handler = logging.FileHandler(log_file, encoding="UTF-8")
file_handler.setFormatter(formatter)
file_handler.setLevel(logging.DEBUG)
stream_handler = logging.StreamHandler()
stream_handler.setFormatter(formatter)
stream_handler.setLevel(logging.INFO)
# xmind to testcase logger
root_logger = logging.getLogger()
root_logger.addHandler(file_handler)
root_logger.addHandler(stream_handler)
root_logger.setLevel(logging.DEBUG)
# flask and werkzeug logger
werkzeug_logger = logging.getLogger("werkzeug")
werkzeug_logger.addHandler(file_handler)
werkzeug_logger.addHandler(stream_handler)
werkzeug_logger.setLevel(logging.DEBUG)

# global variable
UPLOAD_FOLDER = os.path.join(here, "uploads")
ALLOWED_EXTENSIONS = ["xmind"]
DEBUG = True
DATABASE = os.path.join(here, "data.db3")
HOST = "0.0.0.0"

# flask app
app = Flask(__name__)
app.config.from_object(__name__)
app.secret_key = os.urandom(32)


def connect_db():
    return sqlite3.connect(app.config["DATABASE"])


def init_db():
    with closing(connect_db()) as db:
        with app.open_resource("schema.sql", mode="r") as f:
            db.cursor().executescript(f.read())
        db.commit()


def init():
    app.logger.info("Start initializing the database...")
    if not exists(UPLOAD_FOLDER):
        os.mkdir(UPLOAD_FOLDER)

    if not exists(DATABASE):
        app.logger.info("test")
        init_db()
    app.logger.info("Congratulations! the xmind2testcasezen webtool database has initialized successfully!")


@app.before_request
def before_request():
    g.db = connect_db()


@app.teardown_request
def teardown_request(exception):
    db = getattr(g, "db", None)
    if db is not None:
        db.close()


def insert_record(xmind_name, note=""):
    c = g.db.cursor()
    now = str(arrow.now())
    sql = "INSERT INTO records (name,create_on,note) VALUES (?,?,?)"
    c.execute(sql, (xmind_name, now, str(note)))
    g.db.commit()


def delete_record(filename, record_id):
    xmind_file = join(app.config["UPLOAD_FOLDER"], filename)
    testlink_file = join(app.config["UPLOAD_FOLDER"], filename[:-5] + "xml")
    zentao_file = join(app.config["UPLOAD_FOLDER"], filename[:-5] + "csv")

    for f in [xmind_file, testlink_file, zentao_file]:
        if exists(f):
            os.remove(f)

    c = g.db.cursor()
    sql = "UPDATE records SET is_deleted=1 WHERE id = ?"
    c.execute(sql, (record_id,))
    g.db.commit()


def delete_records(keep=20):
    """Clean up files on server and mark the record as deleted"""
    sql = "SELECT * from records where is_deleted<>1 ORDER BY id desc LIMIT -1 offset {}".format(keep)
    assert isinstance(g.db, sqlite3.Connection)
    c = g.db.cursor()
    c.execute(sql)
    rows = c.fetchall()
    for row in rows:
        name = row[1]
        xmind_file = join(app.config["UPLOAD_FOLDER"], name)
        testlink_file = join(app.config["UPLOAD_FOLDER"], name[:-5] + "xml")
        zentao_file = join(app.config["UPLOAD_FOLDER"], name[:-5] + "csv")

        for f in [xmind_file, testlink_file, zentao_file]:
            if exists(f):
                os.remove(f)

        sql = "UPDATE records SET is_deleted=1 WHERE id = ?"
        c.execute(sql, (row[0],))
        g.db.commit()


def get_latest_record():
    found = list(get_records(1))
    if found:
        return found[0]


def get_records(limit=8):
    short_name_length = 120
    c = g.db.cursor()
    sql = "select * from records where is_deleted<>1 order by id desc limit {}".format(int(limit))
    c.execute(sql)
    rows = c.fetchall()

    for row in rows:
        name, short_name, create_on, note, record_id = row[1], row[1], row[2], row[3], row[0]

        # shorten the name for display
        if len(name) > short_name_length:
            short_name = name[:short_name_length] + "..."

        # more readable time format
        create_on = arrow.get(create_on).humanize()
        yield short_name, name, create_on, note, record_id


def allowed_file(filename):
    return "." in filename and filename.rsplit(".", 1)[1] in ALLOWED_EXTENSIONS


def check_file_name(name):
    secured = secure_filename(name)
    if not secured:
        secured = re.sub("[^\w\d]+", "_", name)  # only keep letters and digits from file name
        assert secured, "Unable to parse file name: {}!".format(name)
    return secured + ".xmind"


def save_file(file):
    if file and allowed_file(file.filename):
        # filename = check_file_name(file.filename[:-6])
        filename = file.filename
        upload_to = join(app.config["UPLOAD_FOLDER"], filename)

        if exists(upload_to):
            filename = "{}_{}.xmind".format(filename[:-6], arrow.now().strftime("%Y%m%d_%H%M%S"))
            upload_to = join(app.config["UPLOAD_FOLDER"], filename)

        file.save(upload_to)
        insert_record(filename)
        g.is_success = True
        return filename

    elif file.filename == "":
        g.is_success = False
        g.error = "Please select a file!"

    else:
        g.is_success = False
        g.invalid_files.append(file.filename)


def verify_uploaded_files(files):
    # download the xml directly if only 1 file uploaded
    if len(files) == 1 and getattr(g, "is_success", False):
        g.download_xml = get_latest_record()[1]

    if g.invalid_files:
        g.error = "Invalid file: {}".format(",".join(g.invalid_files))


@app.route("/", methods=["GET", "POST"])
def index(download_xml=None):
    g.invalid_files = []
    g.error = None
    g.download_xml = download_xml
    g.filename = None

    if request.method == "POST":
        if "file" not in request.files:
            return redirect(request.url)

        file = request.files["file"]

        if file.filename == "":
            return redirect(request.url)

        g.filename = save_file(file)
        verify_uploaded_files([file])
        delete_records()

    else:
        g.upload_form = True

    if g.filename:
        return redirect(url_for("preview_file", filename=g.filename))
    else:
        return render_template("index.html", records=list(get_records()))


@app.route("/uploads/<filename>")
def uploaded_file(filename):
    return send_from_directory(app.config["UPLOAD_FOLDER"], filename)


@app.route("/<filename>/to/testlink")
def download_testlink_file(filename):
    full_path = join(app.config["UPLOAD_FOLDER"], filename)

    if not exists(full_path):
        abort(404)

    testlink_xmls_file = xmind_to_testlink_xml_file(full_path)
    filename = os.path.basename(testlink_xmls_file) if testlink_xmls_file else abort(404)

    return send_from_directory(app.config["UPLOAD_FOLDER"], filename, as_attachment=True)


@app.route("/<filename>/to/zentao")
def download_zentao_file(filename):
    full_path = join(app.config["UPLOAD_FOLDER"], filename)

    if not exists(full_path):
        abort(404)

    zentao_csv_file = xmind_to_zentao_csv_file(full_path)
    filename = os.path.basename(zentao_csv_file) if zentao_csv_file else abort(404)

    return send_from_directory(app.config["UPLOAD_FOLDER"], filename, as_attachment=True)


@app.route("/preview/<filename>")
def preview_file(filename):
    full_path = join(app.config["UPLOAD_FOLDER"], filename)

    if not exists(full_path):
        abort(404)
    suite_count = count_testsuits(full_path)
    testcases = get_testcase_list(full_path)

    return render_template("preview.html", name=filename, suite=testcases, suite_count=suite_count)


@app.route("/delete/<filename>/<int:record_id>")
def delete_file(filename, record_id):
    full_path = join(app.config["UPLOAD_FOLDER"], filename)
    if not exists(full_path):
        abort(404)
    else:
        delete_record(filename, record_id)
    return redirect("/")


@app.errorhandler(Exception)
def app_error(e):
    logging.exception(f"error:{e}")
    return str(e)


def launch(host=HOST, debug=True, port=5001):
    init()  # initializing the database
    app.run(host=host, debug=debug, port=port)


if __name__ == "__main__":
    init()  # initializing the database
    app.run(HOST, debug=DEBUG, port=5002)



#!/usr/bin/env python
# _*_ coding:utf-8 _*_

"""
testlink.testlink
"""
import re
from typing import Match, Union

from xmind2testcasezen import const


class TestSuite(object):
    def __init__(self, name="", details="", testcase_list=None, sub_suites=None, statistics=None):
        """
        TestSuite
        :param name: test suite name  > topic title
        :param details: test suite detail infomation > topic notes
        :param testcase_list: test case list
        :param sub_suites: sub test suite list
        :param statistics: testsuite statistics info {'case_num': 0, 'non_execution': 0, 'pass': 0, 'failed': 0, 'blocked': 0, 'skipped': 0}
        """
        self.name = name
        self.details = details
        self.testcase_list = testcase_list
        self.sub_suites = sub_suites  # ignore
        self.statistics = statistics  # ignore

    def to_dict(self):
        data = {"name": self.name, "details": self.details, "testcase_list": [], "sub_suites": []}

        if self.sub_suites:
            for suite in self.sub_suites:
                data["sub_suites"].append(suite.to_dict())

        if self.testcase_list:
            for case in self.testcase_list:
                data["testcase_list"].append(case.to_dict())

        if self.statistics:
            data["statistics"] = self.statistics

        return data


class TestCase(object):
    def __init__(
        self,
        name="",
        category="",
        version=1,
        summary="",
        preconditions="",
        execution_type=1,
        importance=2,
        estimated_exec_duration=3,
        status=7,
        result=0,
        steps=None,
    ):
        """
        TestCase
        :param name: test case name  > topic title
        :param version: test case version infomation
        :param summary: test case summary infomation
        :param preconditions: test case pre condition
        :param execution_type: manual:1 or automate:2
        :param importance: high:1, middle:2, low:3
        :param estimated_exec_duration: estimated execution duration
        :param status: draft:1, ready ro review:2, review in progress:3, rework:4, obsolete:5, future:6, final:7
        :param result: non-execution:0, pass:1, failed:2, blocked:3, skipped:4
        :param steps: test case step list
        """

        self.name = name
        self.version = version
        self.category = category
        self.summary = summary
        self.preconditions = preconditions
        self.execution_type = execution_type
        self.importance = importance
        self.estimated_exec_duration = estimated_exec_duration
        self.status = status
        self.result = result
        self.steps = steps

    def to_dict(self):
        data = {
            "name": self.name,
            "version": self.version,  # TODO(devin): get version content
            "summary": self.summary,
            "category": self.category,
            "preconditions": self.preconditions,
            "execution_type": self.execution_type,
            "importance": self.importance,
            "estimated_exec_duration": self.estimated_exec_duration,  # TODO(devin): get estimated content
            "status": self.status,  # TODO(devin): get status content
            "result": self.result,
            "steps": [],
        }

        if self.steps:
            for step in self.steps:
                data["steps"].append(step.to_dict())

        return data


class TestStep(object):
    def __init__(self, step_number=1, actions="", expectedresults="", execution_type=1, result=0):
        """
        TestStep
        :param step_number: test step number
        :param actions: test step actions
        :param expectedresults: test step expected results
        :param execution_type: test step execution type
        :param result: non-execution:0, pass:1, failed:2, blocked:3, skipped:4
        """
        self.step_number = step_number
        self.actions = actions
        self.expectedresults = expectedresults
        self.execution_type = execution_type  # TODO(devin): get execution type content
        self.result = result

    def to_dict(self):
        data = {
            "step_number": self.step_number,
            "actions": self.actions,
            "expectedresults": self.expectedresults,
            "execution_type": self.execution_type,
            "result": self.result,
        }

        return data


class AttachedTopicAttribute:
    def __init__(self, attached_topic):
        self.id: str = attached_topic.get("id", "")
        self.title: str = attached_topic.get("title", "")
        self.note: str = self.get_notes(attached_topic.get("notes", ""))
        self.href: str = attached_topic.get("href", "")
        self.markers: list = self.get_markers(attached_topic.get("markers", ""))
        self.labels: list = attached_topic.get("labels", [])
        self.image: list = self.get_image(attached_topic.get("image", ""))
        self.sub_attached_topics: list = self.get_sub_attached_topics(attached_topic.get("children", {}))
        self.execution_type: int = self.get_execution_type(self.labels, self.markers)
        self.is_ignore: bool = self.get_attached_topic_type_by_labels(const.TESTCASE_IGNORE_TAGS, self.labels)
        # 测试用例 执行步骤 预期结果 前置条件 不允许同时关联同一节点
        self.is_valid: bool = self.get_valid_testcase_attached_topic(self.labels)
        self.is_testcase: bool = self.get_attached_topic_type_by_labels(const.TESTCASE_TAG, self.labels)
        self.is_precondition: bool = self.get_attached_topic_type_by_labels(const.PRECONDITIONS_TAG, self.labels)
        self.is_teststep: bool = self.get_attached_topic_type_by_labels(const.TESTSTEP_TAG, self.labels)
        self.is_expect_result: bool = self.get_attached_topic_type_by_labels(const.EXPECT_RESULT_TAG, self.labels)
        self.importance: int = self.get_importance_by_labels(const.IMPORTANCE_TAGS, self.labels, self.markers)

    def get_notes(self, notes):
        notes_content = ""
        if notes:
            notes_content = notes.get("plain").get("content")
        return notes_content

    def get_image(self, image):
        src = ""
        if image:
            src = image.get("src", "")
        return src

    def get_valid_testcase_attached_topic(self, labels):
        cou = 0
        labels = [label.lower() for label in labels]
        if const.TESTCASE_TAG.lower() in labels:
            cou += 1
        if const.TESTSTEP_TAG.lower() in labels:
            cou += 1
        if const.EXPECT_RESULT_TAG.lower() in labels:
            cou += 1
        if const.PRECONDITIONS_TAG.lower() in labels:
            cou += 1

        return cou <= 1

    def get_sub_attached_topics(self, children):
        cur_attached_topics = []
        if children:
            cur_attached_topics = children.get("attached", [])
        return cur_attached_topics

    def get_markers(self, markers):
        cur_markers = []
        if markers:
            cur_markers = [marker.get("markerId", "") for marker in markers]
        return cur_markers

    def get_attached_topic_type_by_labels(self, tag, labels):
        tags = []
        if isinstance(tag, list):
            tags.extend(tag)
        else:
            tags.append(tag)
        return self.match_tags(tags, labels)
        # if tag in labels:
        #     return True
        # return False

    def get_importance_by_labels(self, tags, labels, makers=list()) -> int:
        importance = 2
        labels.extend(makers)
        # logging.info(f'AAAAAA-labels:{labels}')
        match = self.match_tags(tags, labels, makers, is_regex=True)
        if match:
            importance = int(re.findall("[0-9]", match.string)[0])
        return importance

    def get_execution_type(self, labels=[], markers=[]):
        execution_type = const.EXECUTION_MANUAL_TYPE
        is_match = self.match_tags(const.EXECUTION_AUTO_TYPE_TAG, labels, markers)
        if is_match:
            execution_type = const.EXECUTION_AUTO_TYPE
        return execution_type

    def match_tags(self, tags, labels=[], markers=[], is_regex=False) -> Union[Match[str], None, bool]:
        labels.extend(markers)
        labels = [label.lower() for label in labels]
        tags = [tag.lower() for tag in tags]
        if is_regex:
            for label in labels:
                for tag in tags:
                    match = re.match(tag, label)
                    if match:
                        return match
            return None
        else:
            for tag in tags:
                if tag in labels:
                    return True
            return False

    def get_preconditions(self):
        preconditions = []
        if not self.is_precondition:
            return preconditions

        if not self.sub_attached_topics:
            preconditions.append(self.title)

        else:
            for sub_attached_topic in self.sub_attached_topics:
                sub_attached_topic = AttachedTopicAttribute(sub_attached_topic)
                preconditions.append(sub_attached_topic.title)

        return preconditions



import json
from zipfile import ZipFile

from . import cache, config

content_json = "content.json"


def open_xmind(file_path):
    """open xmind as zip file and cache the content."""
    cache.clear()
    with ZipFile(file_path) as xmind:
        for f in xmind.namelist():
            for key in [content_json]:
                if f == key:
                    cache[key] = xmind.open(f).read().decode("utf-8")


def get_sheets():
    """get all sheet as generator and yield."""
    for sheet in json.loads(cache[content_json]):
        yield sheet


def sheet_to_dict(sheet):
    """convert a sheet to dict type."""
    topic = sheet["rootTopic"]
    result = {"title": sheet["title"], "topic": node_to_dict(topic), "structure": get_sheet_structure(sheet)}

    if config["showTopicId"]:
        result["id"] = sheet["id"]

    if config["hideEmptyValue"]:
        result = {k: v for k, v in result.items() if v}

    return result


def get_sheet_structure(sheet):
    root_topic = sheet["rootTopic"]
    return root_topic.get("structureClass", None)


def node_to_dict(node):
    """parse Element to dict data type."""
    child = children_topics_of(node)

    d = {
        "id": "",
        "title": node.get("title", ""),
        "note": note_of(node),
        "markers": maker_of(node),
        "label": labels_of(node),
        "link": link_of(node),
        "image": image_of(node),
        "comment": None
        # 'callout': callout_of(node)
    }

    if d["link"]:
        if d["link"].startswith("xmind"):
            d["link"] = "[To another xmind topic!]"

        if d["link"].startswith("xap:attachments"):
            del d["link"]
            d["title"] = "[Attachment]{0}".format(d["title"])

    if child:
        d["topics"] = []
        for c in child:
            d["topics"].append(node_to_dict(c))

    if config["showTopicId"]:
        d["id"] = node["id"]

    # if config['hideEmptyValue']:
    # d = {k: v for k, v in d.items() if v or k == 'title'}

    return d


def children_topics_of(topic_node):
    children = topic_node.get("children", None)

    if children:
        return children.get("attached", None)


def link_of(node):
    return node.get("href", None)


def image_of(node):
    return node.get("image", None)


def labels_of(node):
    return node.get("labels", None)


def note_of(node):
    note_node = node.get("notes", None)

    if note_node:
        note = note_node.get("plain", None)
        if note:
            return note.get("content", "").strip()


def maker_of(topic_node):
    makers = []
    maker_node = topic_node.get("markers", None)
    if maker_node is not None:
        for maker in maker_node:
            makers.append(maker.get("markerId", None))

    return makers


def callout_of(topic_node):
    callout = topic_node.get("children", None)
    if callout:
        callout = callout.get("callout", None)
        if callout:
            return [x["title"] for x in callout]



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Time:2023/3/1 19:07
# @Author:boyizhang
import json
import logging
import os

from xmind2testcasezen import const
from xmind2testcasezen.parse_new import ParseNew
from xmind2testcasezen.parse_old import ParseOld
from xmind2testcasezen.utils import get_absolute_path, parser_xmind_to_dict


def get_raw_testsuites(xmind_file):
    """Load the XMind file and parse to `xmind2testcasezen.metadata.TestSuite` list"""
    xmind_file = get_absolute_path(xmind_file)
    # 根据解析对象使用指定的入口函数解析
    parser_type, xmind_content_dict = parser_xmind_to_dict(xmind_file)
    if parser_type == const.TAG_JSON:
        logging.info("parser new version xmind")
        parse = ParseNew()
    elif parser_type == const.TAG_PRO:
        logging.info("parser new version xmind pro")
        parse = ParseOld()
    else:
        logging.info("parser old version xmind")
        parse = ParseOld()

    logging.info(f"xmind_content_dict:{json.dumps(xmind_content_dict)}")

    if xmind_content_dict:
        try:
            testsuites = parse.parse_testsuites(xmind_content_dict)
            return testsuites
        except Exception as e:
            logging.exception(f"e:{e}")

    else:
        logging.error("Invalid XMind file(%s): it is empty!", xmind_file)
        return []


def count_testsuits(xmind_file):
    testsuites = get_raw_testsuites(xmind_file)
    suite_count = 0
    for suite in testsuites:
        suite_count += len(suite.sub_suites)

    return suite_count


def get_testsuite_list(xmind_file):
    """Load the XMind file and get all testsuite in it

    :param xmind_file: the target XMind file
    :return: a list of testsuite data
    """
    xmind_file = get_absolute_path(xmind_file)
    logging.info("Start converting XMind file(%s) to testsuite data list...", xmind_file)
    testsuite_list = get_raw_testsuites(xmind_file)
    suite_data_list = []

    for testsuite in testsuite_list:
        # product_statistics = {'case_num': 0, 'non_execution': 0, 'pass': 0, 'failed': 0, 'blocked': 0, 'skipped': 0}
        # for sub_suite in testsuite.sub_suites:
        #     suite_statistics = {'case_num': len(sub_suite.testcase_list), 'non_execution': 0, 'pass': 0, 'failed': 0,
        #                         'blocked': 0, 'skipped': 0}
        #     for case in sub_suite.testcase_list:
        #         if case.result == 0:
        #             suite_statistics['non_execution'] += 1
        #         elif case.result == 1:
        #             suite_statistics['pass'] += 1
        #         elif case.result == 2:
        #             suite_statistics['failed'] += 1
        #         elif case.result == 3:
        #             suite_statistics['blocked'] += 1
        #         elif case.result == 4:
        #             suite_statistics['skipped'] += 1
        #         else:
        #             logging.warning('This testcase result is abnormal: %s, please check it: %s', case.result,
        #                             case.to_dict())
        #     sub_suite.statistics = suite_statistics
        #     for item in product_statistics:
        #         product_statistics[item] += suite_statistics[item]
        #
        # testsuite.statistics = product_statistics
        suite_data = testsuite.to_dict()
        suite_data_list.append(suite_data)

    logging.info("Convert XMind file(%s) to testsuite data list successfully!", xmind_file)
    return suite_data_list


def get_testcase_list(xmind_file):
    """Load the XMind file and get all testcase in it

    :param xmind_file: the target XMind file
    :return: a list of testcase data
    """
    xmind_file = get_absolute_path(xmind_file)
    logging.info("Start converting XMind file(%s) to testcases dict data...", xmind_file)
    testsuites = get_raw_testsuites(xmind_file)
    testcases = []

    for testsuite in testsuites:
        product = testsuite.name
        for suite in testsuite.sub_suites:
            for case in suite.testcase_list:
                case_data = case.to_dict()
                case_data["product"] = product
                case_data["suite"] = f"{product}::{suite.name}"
                testcases.append(case_data)

    logging.info("Convert XMind file(%s) to testcases dict data successfully!", xmind_file)
    return testcases


def gen_testsuite_to_json_file(xmind_file):
    """Convert XMind file to a testsuite json file"""
    xmind_file = get_absolute_path(xmind_file)
    logging.info("Start converting XMind file(%s) to testsuites json file...", xmind_file)
    testsuites = get_testsuite_list(xmind_file)
    testsuite_json_file = xmind_file[:-6] + "_testsuite.json"

    if os.path.exists(testsuite_json_file):
        os.remove(testsuite_json_file)
        # logging.info('The testsuite json file already exists, return it directly: %s', testsuite_json_file)
        # return testsuite_json_file

    with open(testsuite_json_file, "w", encoding="utf8") as f:
        f.write(json.dumps(testsuites, indent=4, separators=(",", ": "), ensure_ascii=False))
        logging.info(
            "Convert XMind file(%s) to a testsuite json file(%s) successfully!", xmind_file, testsuite_json_file
        )

    return testsuite_json_file


def gen_testcase_to_json_file(xmind_file):
    """Convert XMind file to a testcase json file"""
    xmind_file = get_absolute_path(xmind_file)
    logging.info("Start converting XMind file(%s) to testcases json file...", xmind_file)
    testcases = get_testcase_list(xmind_file)
    testcase_json_file = xmind_file[:-6] + ".json"
    logging.info(f"testcase_json_file:{testcase_json_file}")
    if os.path.exists(testcase_json_file):
        os.remove(testcase_json_file)
        logging.info("The testcase json file already exists, return it directly: %s", testcase_json_file)
        # return testcase_json_file

    with open(testcase_json_file, "w", encoding="utf8") as f:
        f.write(json.dumps(testcases, indent=4, separators=(",", ": "), ensure_ascii=False))
        logging.info("Convert XMind file(%s) to a testcase json file(%s) successfully!", xmind_file, testcase_json_file)

    return testcase_json_file


if __name__ == "__main__":
    # gen_testcase_to_json_file('/Users/boyizhang/Downloads/new-[SPSOB-2252][Package] BP V3.4.1 Auto-pricing for special discount.xmind')
    gen_testsuite_to_json_file(
        "/Users/boyizhang/Downloads/new-[SPSOB-2252][Package] BP V3.4.1 Auto-pricing for special discount.xmind"
    )



#!/usr/env/bin python
# -*- coding: utf-8 -*-
import io
import os
import sys
from shutil import rmtree

from setuptools import Command, find_packages, setup

about = {}
here = os.path.abspath(os.path.dirname(__file__))
with io.open(os.path.join(here, "xmind2testcasezen", "__about__.py"), encoding="utf-8") as f:  # custom
    exec(f.read(), about)

with io.open("README.md", encoding="utf-8") as f:
    long_description = f.read()

install_requires = [  # custom
    "xmind",
    "flask",
    "arrow",
]


class PyPiCommand(Command):
    """Build and publish this package and make a tag.
    Support: python setup.py pypi
    Copied from requests_html
    """

    user_options = []

    @staticmethod
    def status(s):
        """Prints things in green color."""
        print("\033[0;32m{0}\033[0m".format(s))

    def initialize_options(self):
        """override"""

    def finalize_options(self):
        """override"""

    def run(self):
        self.status("Building Source and Wheel (universal) distribution...")
        os.system("{0} setup.py sdist bdist_wheel --universal".format(sys.executable))

        self.status("Uploading the package to PyPi via Twine...")
        os.system("twine upload dist/*")

        self.status("Publishing git tags...")
        os.system("git tag v{0}".format(about["__version__"]))
        os.system("git push --tags")

        try:
            self.status("Removing current build artifacts...")
            rmtree(os.path.join(here, "dist"))
            rmtree(os.path.join(here, "build"))
            rmtree(os.path.join(here, "xmind2testcasezen.egg-info"))  # custom
        except OSError:
            pass

        self.status("Congratulations! Upload PyPi and publish git tag successfully...")
        sys.exit()


setup(
    name=about["__title__"],
    version=about["__version__"],
    description=about["__description__"],
    long_description=long_description,
    long_description_content_type="text/markdown",
    keywords=about["__keywords__"],
    author=about["__author__"],
    author_email=about["__author_email__"],
    url=about["__url__"],
    license=about["__license__"],
    packages=find_packages(exclude=["tests", "test.*", "docs"]),  # custom
    package_data={  # custom
        "": ["README.md"],
        "webtool": ["static/*", "static/css/*", "static/guide/*", "templates/*", "schema.sql"],
    },
    install_requires=install_requires,
    extras_require={},
    python_requires=">=3.0, <4",  # custom
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    entry_points={  # custom
        "console_scripts": [
            "xmind2testcasezen=xmind2testcasezen.cli:cli_main",
        ]
    },
    cmdclass={
        # python3 setup.py pypi
        "pypi": PyPiCommand
    },
)



#!/usr/bin/env python
# _*_ coding:utf-8 _*_


TAG_XML = "xml"
TAG_JSON = "json"
TAG_PRO = "zen_pro"

TAG_TESTSUITE = "testsuite"
TAG_DETAILS = "details"

TAG_TESTCASE = "testcase"
TAG_VERSION = "version"
TAG_SUMMARY = "summary"
TAG_PRECONDITIONS = "preconditions"
TAG_IMPORTANCE = "importance"
TAG_ESTIMATED_EXEC_DURATION = "estimated_exec_duration"
TAG_STATUS = "status"
TAG_IS_OPEN = "is_open"
TAG_ACTIVE = "active"
TAG_STEPS = "steps"
TAG_STEP = "step"
TAG_STEP_NUMBER = "step_number"
TAG_ACTIONS = "actions"
TAG_EXPECTEDRESULTS = "expectedresults"
TAG_EXECUTION_TYPE = "execution_type"

ATTR_NMAE = "name"
ATTR_ID = "id"
ATTR_INTERNALID = "internalid"

# -- boyi --
PRECONDITIONS_TAG = "前置条件"
TESTCASE_TAG = "测试用例"
TESTCASE_IGNORE_TAGS = ["ignore", "skipped"]
TESTSTEP_TAG = "执行步骤"
EXPECT_RESULT_TAG = "预期结果"

IMPORTANCE_TAGS = ["P[0-9]", "priority-[0-9]"]

EXECUTION_AUTO_TYPE_TAG = ["自动", "auto", "automate", "automation"]
EXECUTION_MANUAL_TYPE_TAG = ["手动", "手工", "manual"]
EXECUTION_MANUAL_TYPE = 1
EXECUTION_AUTO_TYPE = 2

config = {
    "sep": '<font color="red"> >> </font>',
    "valid_sep": "&>+/-",
    "precondition_sep": "\n----\n",
    "summary_sep": "\n----\n",
    "step": "\n#####\n",
    "ignore_char": "#!！",
}
